@@ -1,4 +1,4 @@
-# Copyright 2014 Microsoft Corporation
+# Copyright 2018 Microsoft Corporation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# Requires Python 2.4+ and Openssl 1.0+
+# Requires Python 2.6+ and Openssl 1.0+
 #
 
 """
@@ -22,6 +22,7 @@ Provision handler
 import os
 import os.path
 import re
+import time
 
 from datetime import datetime
 
@@ -53,16 +54,16 @@ class ProvisionHandler(object):
         self.protocol_util = get_protocol_util()
 
     def run(self):
-        # If provision is not enabled, report ready and then return
         if not conf.get_provision_enabled():
             logger.info("Provisioning is disabled, skipping.")
+            self.write_provisioned()
+            self.report_ready()
             return
 
         try:
             utc_start = datetime.utcnow()
             thumbprint = None
 
-            # if provisioning is already done, return
             if self.is_provisioned():
                 logger.info("Provisioning already completed, skipping.")
                 return
@@ -76,7 +77,7 @@ class ProvisionHandler(object):
             logger.info("Copying ovf-env.xml")
             ovf_env = self.protocol_util.copy_ovf_env()
 
-            self.protocol_util.get_protocol_by_file()
+            self.protocol_util.get_protocol(by_file=True)
             self.report_not_ready("Provisioning", "Starting")
             logger.info("Starting provisioning")
 
@@ -85,49 +86,72 @@ class ProvisionHandler(object):
             thumbprint = self.reg_ssh_host_key()
             self.osutil.restart_ssh_service()
 
-            # write out provisioned file and report Ready
             self.write_provisioned()
 
-            self.report_event("Provision succeed",
+            self.report_event("Provisioning succeeded ({0}s)".format(self._get_uptime_seconds()),
                 is_success=True,
                 duration=elapsed_milliseconds(utc_start))
 
+            self.handle_provision_guest_agent(ovf_env.provision_guest_agent)
+
             self.report_ready(thumbprint)
             logger.info("Provisioning complete")
 
         except (ProtocolError, ProvisionError) as e:
+            msg = "Provisioning failed: {0} ({1}s)".format(ustr(e), self._get_uptime_seconds())
+            logger.error(msg)
             self.report_not_ready("ProvisioningFailed", ustr(e))
-            self.report_event(ustr(e))
-            logger.error("Provisioning failed: {0}", ustr(e))
+            self.report_event(msg, is_success=False)
             return
 
     @staticmethod
     def validate_cloud_init(is_expected=True):
-        pids = [pid for pid in os.listdir('/proc') if pid.isdigit()]
         is_running = False
+        if os.path.isdir("/proc"):
+            pids = [pid for pid in os.listdir('/proc') if pid.isdigit()]
+        else:
+            pids = []
         for pid in pids:
             try:
-                pname = open(os.path.join('/proc', pid, 'cmdline'), 'rb').read()
-                if CLOUD_INIT_REGEX.match(pname):
-                    is_running = True
-                    msg = "cloud-init is running [PID {0}, {1}]".format(pid,
-                                                                        pname)
-                    if is_expected:
-                        logger.verbose(msg)
-                    else:
-                        logger.error(msg)
-                    break
+                with open(os.path.join('/proc', pid, 'cmdline'), 'rb') as fh:
+                    pname = fh.read()
+                    if CLOUD_INIT_REGEX.match(pname):
+                        is_running = True
+                        msg = "cloud-init is running [PID {0}, {1}]".format(pid,
+                                                                            pname)
+                        if is_expected:
+                            logger.verbose(msg)
+                        else:
+                            logger.error(msg)
+                        break
             except IOError:
                 continue
         return is_running == is_expected
 
+    @staticmethod
+    def _get_uptime_seconds():
+        try:
+            with open('/proc/uptime') as fh:
+                uptime, _ = fh.readline().split()
+                return uptime
+        except:
+            return 0
+
     def reg_ssh_host_key(self):
         keypair_type = conf.get_ssh_host_keypair_type()
         if conf.get_regenerate_ssh_host_key():
             fileutil.rm_files(conf.get_ssh_key_glob())
-            keygen_cmd = "ssh-keygen -N '' -t {0} -f {1}"
-            shellutil.run(keygen_cmd.format(keypair_type,
-                        conf.get_ssh_key_private_path()))
+            if conf.get_ssh_host_keypair_mode() == "auto":
+                '''
+                The -A option generates all supported key types.
+                This is supported since OpenSSH 5.9 (2011).
+                '''
+                shellutil.run("ssh-keygen -A")
+            else:
+                keygen_cmd = "ssh-keygen -N '' -t {0} -f {1}"
+                shellutil.run(keygen_cmd.
+                              format(keypair_type,
+                                     conf.get_ssh_key_private_path()))
         return self.get_ssh_host_key_thumbprint()
 
     def get_ssh_host_key_thumbprint(self, chk_err=True):
@@ -159,7 +183,7 @@ class ProvisionHandler(object):
             return False
 
         s = fileutil.read_file(self.provisioned_file_path()).strip()
-        if s != self.osutil.get_instance_id():
+        if not self.osutil.is_current_instance_id(s):
             if len(s) > 0:
                 logger.warn("VM is provisioned, "
                             "but the VM unique identifier has changed -- "
@@ -170,6 +194,7 @@ class ProvisionHandler(object):
                 deprovision_handler.run_changed_unique_id()
 
             self.write_provisioned()
+            self.report_ready()
 
         return True
 
@@ -178,6 +203,19 @@ class ProvisionHandler(object):
             self.provisioned_file_path(),
             get_osutil().get_instance_id())
 
+    @staticmethod
+    def write_agent_disabled():
+        logger.warn("Disabling guest agent in accordance with ovf-env.xml")
+        fileutil.write_file(conf.get_disable_agent_file_path(), '')
+
+    def handle_provision_guest_agent(self, provision_guest_agent):
+        self.report_event(message=provision_guest_agent,
+                          is_success=True,
+                          duration=0,
+                          operation=WALAEventOperation.ProvisionGuestAgent)
+        if provision_guest_agent and provision_guest_agent.lower() == 'false':
+            self.write_agent_disabled()
+
     def provision(self, ovfenv):
         logger.info("Handle ovf-env.xml.")
         try:
@@ -233,9 +271,14 @@ class ProvisionHandler(object):
         fileutil.write_file(customdata_file, customdata)
 
         if conf.get_execute_customdata():
+            start = time.time()
             logger.info("Execute custom data")
             os.chmod(customdata_file, 0o700)
             shellutil.run(customdata_file)
+            add_event(name=AGENT_NAME,
+                        duration=int(time.time() - start),
+                        is_success=True,
+                        op=WALAEventOperation.CustomData)
 
     def deploy_ssh_pubkeys(self, ovfenv):
         for pubkey in ovfenv.ssh_pubkeys:
@@ -247,12 +290,13 @@ class ProvisionHandler(object):
             logger.info("Deploy ssh key pairs.")
             self.osutil.deploy_ssh_keypair(ovfenv.username, keypair)
 
-    def report_event(self, message, is_success=False, duration=0):
+    def report_event(self, message, is_success=False, duration=0,
+                     operation=WALAEventOperation.Provision):
         add_event(name=AGENT_NAME,
                     message=message,
                     duration=duration,
                     is_success=is_success,
-                    op=WALAEventOperation.Provision)
+                    op=operation)
 
     def report_not_ready(self, sub_status, description):
         status = ProvisionStatus(status="NotReady", subStatus=sub_status,
